import pytest
import json
import os
from time import sleep

from rara_tools.elastic import KataElastic

with open("./tests/test_data/elastic_docs.json") as fh:
    TEST_DOCUMENTS = json.load(fh)

es_url = os.getenv("ELASTIC_TEST_URL", "http://localhost:9200")
ELASTIC = KataElastic(es_url)
ELASTIC_BAD = KataElastic("http://locallost:9012")
TEST_INDEX_NAME = "tools_testing_index"
TEST_DOCUMENT_ID = None


@pytest.mark.order(1)
def test_index_creation_and_data_indexing():
    """ Tests if index created and documents indexed.
    """
    # Create test index
    created = ELASTIC.create_index(TEST_INDEX_NAME)
    assert created["acknowledged"] is True
    # Add test documents
    for document in TEST_DOCUMENTS:
        indexed = ELASTIC.index_document(TEST_INDEX_NAME, document)
        assert indexed["result"] == "created"
    # let it index
    sleep(1)

@pytest.mark.order(2)
def test_check():
    """Tests health check method.
    """
    assert ELASTIC.check() is True
    # test bad connection
    assert ELASTIC_BAD.check() is False

@pytest.mark.order(3)
def test_get_document_by_key():
    """Tests if correct documents fetched.
    """
    result = ELASTIC.get_documents_by_key(TEST_INDEX_NAME, "foo")
    assert len(result) == 2
    result = ELASTIC.get_documents_by_key(TEST_INDEX_NAME, "bar")
    global TEST_DOCUMENT_ID
    TEST_DOCUMENT_ID = result[0].meta.id
    assert len(result) == 1
    result = ELASTIC.get_documents_by_key(TEST_INDEX_NAME, "loll")
    assert len(result) == 0

@pytest.mark.order(5)
def test_document_deleting():
    """ Tests deleting a document from index.
    """
    deleted = ELASTIC.delete_document(TEST_INDEX_NAME, TEST_DOCUMENT_ID)
    assert deleted["result"] == "deleted"
    sleep(1)
    # check if document was actually deleted
    result = ELASTIC.get_documents_by_key(TEST_INDEX_NAME, "bar")
    assert len(result) == 0

@pytest.mark.order(5)
def test_index_deleting():
    """ Tests deleting index. We delete the test index now.
    """
    deleted = ELASTIC.delete_index(TEST_INDEX_NAME)
    assert deleted["acknowledged"] is True



from typing import Dict, Optional, List
from elasticsearch import Elasticsearch
from elasticsearch_dsl import Search

from .decorators import _elastic_connection


class KataElastic:
    """A class to manage all required Elasticsearch operations for Kata.
    """
    def __init__(self, elasticsearch_url: str, timeout: Optional[int] = None):
        self.timeout = timeout
        self.elasticsearch_url = elasticsearch_url
        self.elasticsearch = Elasticsearch(self.elasticsearch_url, request_timeout=self.timeout)

    @_elastic_connection
    def check(self) -> bool:
        """Checks Elasticsearch connection.
        :return: bool: Elasticsearch alive or dead.
        """
        if self.elasticsearch.ping():
            return True
        return False

    @_elastic_connection
    def create_index(
            self,
            index: str,
            shards: int = 3,
            replicas: int = 1,
            settings: Optional[dict] = None
    ) -> Dict:
        """Creates empty index.
        :param: index str: Name of the index to create.
        :param: shards int: Number of shards for the index.
        :param: replicas int: Number of replicas of the index.
        :param: settings dict: Overwrite settings for the index.
        """
        body = settings or {
            "number_of_shards": shards,
            "number_of_replicas": replicas,
        }
        return self.elasticsearch.indices.create(index=index, settings=body)

    @_elastic_connection
    def delete_index(self, index: str, ignore: Optional[bool] = True) -> Dict:
        """Deletes index.
        :param: index str: Name of the index to be deleted.
        :param: ignore bool: Ignore errors because of closed/deleted index.
        :return: Dict of Elastic's acknowledgement of the action.
        """
        response = self.elasticsearch.indices.delete(index=index, ignore_unavailable=ignore)
        return response

    @_elastic_connection
    def delete_document(self, index: str, document_id: str) -> Dict:
        """Deletes document fom index.
        :param: document_id str: ID of the document to be deleted.
        :param: index str: Index where the document is to be found.
        :param: ignore bool: Ignore errors because of closed/deleted index.
        :return: Dict of Elastic's acknowledgement of the action.
        """
        response = self.elasticsearch.delete(id=document_id, index=index)
        return response

    @_elastic_connection
    def index_document(self, index: str, body: dict, document_id: Optional[str] = None) -> Dict:
        """Indexes document.
        :param: index str: Index that document will be indexed into.
        :param: body dict: Document body.
        :param: document_id str: Optional id for the document. Is generated automatically if None.
        :return: Dict of Elastic's acknowledgement of the action.
        """
        if document_id:
            indexed = self.elasticsearch.index(index=index, id=document_id, body=body)
        else:
            indexed = self.elasticsearch.index(index=index, body=body)
        return indexed

    @_elastic_connection
    def get_documents_by_key(self, index: str, document_key: str) -> List:
        """This method is for retrieving all texts/pages of the original document.
        :param: index str: Index to search the documents from.
        :param: document_key str: parent_id field that connects pages of document together.
        :return: List of matching documents. 
        """
        s = Search(using=self.elasticsearch, index=index)
        docs = s.query("match", parent_id=document_key).execute()
        return docs

    def __str__(self) -> str:
         return self.elasticsearch_url
